from django.db import models
from kanban.lib import lp_init, get_sp_for_bug

bug_statuses = (
    "New",
    "Incomplete",
    "Opinion",
    "Invalid",
    "Won't Fix",
    "Confirmed",
    "Triaged",
    "In Progress",
    "Fix Committed",
    "Fix Released",
)


class Project(models.Model):
    name = models.CharField(max_length=255)
    title = models.CharField(max_length=255)

    @property
    def milestones(self):
        return self.all_milestones.all()

    def __unicode__(self):
        return self.name


class Milestone(models.Model):
    name = models.CharField(max_length=255)
    title = models.CharField(max_length=255)
    project = models.ForeignKey(Project, related_name='all_milestones')
    max_sp = models.IntegerField()
    can_be_populated = models.BooleanField()
    can_be_migrated = models.BooleanField()
    sort_order = models.SmallIntegerField()

    @property
    def bugs(self):
        return self.all_bugs.all()

    @property
    def total_sp(self):
        total_sp = 0
        for bug in self.bugs:
            total_sp += bug.sp
        return total_sp

    @property
    def is_full(self):
        return self.total_sp >= self.max_sp

    @property
    def is_over_sp(self):
        return self.max_sp > 0 and self.total_sp > self.max_sp

    def auto_populate(self):
        available_sp = self.max_sp - self.total_sp
        lp = lp_init('default')
        p = lp.projects[self.project.name]
        all_tasks = p.searchTasks(
            status=['Confirmed', 'Triaged'],
            importance=['Critical', 'High', 'Medium', 'Low'],
            order_by='-importance')
        for bug_task in all_tasks:
            if available_sp <= 0:
                break
            if bug_task.milestone is not None:
                continue
            bug_task.milestone = self.get_real_lp_milestone(lp)
            bug_task.lp_save()
            bug = self.add_bug(bug_task)
            print bug_task.bug.id
            available_sp -= bug.sp

    def add_bug(self, bug_task):
        b = Bug()
        b.number = bug_task.bug.id
        b.title = bug_task.bug.title
        if bug_task.assignee:
            b.assignee = bug_task.assignee.name
            b.assignee_name = bug_task.assignee.display_name
        b.priority = bug_task.importance
        b.status = bug_task.status
        b.milestone = self
        b.sp = get_sp_for_bug(bug_task.bug)
        b.save()
        return b

    def get_real_lp_milestone(self, lp):
        p = lp.projects[self.project.name]
        all_milestones = p.active_milestones
        for m in all_milestones:
            if m.name == self.name:
                return m
        return None

    def __unicode__(self):
        return "%s (%s)" % (self.title, self.name)


class Bug(models.Model):
    number = models.IntegerField()
    title = models.CharField(max_length=255)
    assignee = models.CharField(max_length=255, blank=True)
    assignee_name = models.CharField(max_length=255, blank=True)
    priority = models.CharField(max_length=255)
    status = models.CharField(max_length=255)
    milestone = models.ForeignKey(Milestone, related_name='all_bugs')
    sp = models.FloatField()

    def status_class(self):
        if self.status == "Incomplete":
            return "blocked"
        else:
            return ""

    def __unicode__(self):
        return "%s %s (%s)" % (self.number, self.title, self.assignee)
